@@ -210,173 +210,7 @@ sub _send_request
 						}
 					}
 					my $response_packet_payload = $self->{ipcon}->_get_payload_from_data($response_packet);
-					my @form_return_arr = split(' ', $form_return);
-
-					if(scalar(@form_return_arr) > 1)
-					{
-						my @copy_info_arr;
-						my $copy_info_arr_len;
-						my $copy_from_index = 0;
-						my $arguments_arr_index = 0;
-						my $anon_arr_index = 0;
-						my @return_arr;
-						my @arguments_arr = unpack('('.$form_return.')<', $response_packet_payload);
-
-						foreach(@form_return_arr)
-						{
-							my $count = $_;
-							my @form_single_arr = split('', $_);
-
-							if($form_single_arr[0] eq 'c' ||
-							   $form_single_arr[0] eq 'C' ||
-							   $form_single_arr[0] eq 's' ||
-							   $form_single_arr[0] eq 'S' ||
-							   $form_single_arr[0] eq 'l' ||
-							   $form_single_arr[0] eq 'L' ||
-							   $form_single_arr[0] eq 'q' ||
-							   $form_single_arr[0] eq 'Q' ||
-							   $form_single_arr[0] eq 'f')
-							{
-								if(scalar(@form_single_arr) > 1)
-								{
-									$count =~ s/[^\d]//g;
-									push(@copy_info_arr, [$copy_from_index, $count]);
-									$copy_from_index += $count;
-
-									next;
-								}
-								else
-								{
-									$copy_from_index ++;
-								}
-							}
-							else
-							{
-								$copy_from_index ++;
-							}
-						}
-
-						$copy_info_arr_len = scalar(@copy_info_arr);
-
-						if($copy_info_arr_len > 0)
-						{
-							while($arguments_arr_index < scalar(@arguments_arr))
-							{
-								if(scalar(@copy_info_arr) > 0)
-								{
-									if($copy_info_arr[0][0] == $arguments_arr_index)
-									{
-										my $anon_arr_iterator = $copy_info_arr[0][0];
-
-										if($anon_arr_index == 0)
-										{
-											$anon_arr_index = $anon_arr_iterator;
-										}
-
-										$return_arr[$anon_arr_index] = [];
-
-										for(; $anon_arr_iterator < ($copy_info_arr[0][0]+$copy_info_arr[0][1]); $anon_arr_iterator++)
-										{
-											push($return_arr[$anon_arr_index], $arguments_arr[$anon_arr_iterator]);
-										}
-
-										$anon_arr_index ++;
-										$arguments_arr_index += $copy_info_arr[0][1];
-										shift(@copy_info_arr);
-									}
-									else
-									{
-										push(@return_arr, $arguments_arr[$arguments_arr_index]);
-										$anon_arr_index ++;
-										$arguments_arr_index ++;
-									}
-								}
-								else
-								{
-									push(@return_arr, $arguments_arr[$arguments_arr_index]);
-									$arguments_arr_index ++;
-								}
-							}
-
-							my $i = 0;
-
-							foreach(@form_return_arr)
-							{
-								my @form_single_arr = split('', $_);
-
-								if($form_single_arr[0] eq 'a' && scalar(@form_single_arr) > 1)
-								{
-									my @string_to_split_arr = split('', $return_arr[$i]);
-									$return_arr[$i] = [];
-
-									foreach(@string_to_split_arr)
-									{
-										push($return_arr[$i], $_);
-									}
-								}
-
-								$i++;
-							}
-
-							return @return_arr;
-						}
-
-						if($copy_info_arr_len == 0)
-						{
-							my @return_arr = unpack($form_return, $response_packet_payload);
-							my $i = 0;
-
-							foreach(@form_return_arr)
-							{
-								my @form_single_arr = split('', $_);
-
-								if($form_single_arr[0] eq 'a' && scalar(@form_single_arr) > 1)
-								{
-									my @string_to_split_arr = split('', $return_arr[$i]);
-									$return_arr[$i] = [];
-
-									foreach(@string_to_split_arr)
-									{
-										push($return_arr[$i], $_);
-									}
-								}
-
-								$i++;
-							}
-
-							return @return_arr;
-						}
-					}
-
-					if(scalar(@form_return_arr) == 1)
-					{
-						my @form_return_arr_0_arr = split('', $form_return_arr[0]);
-
-						if(scalar(@form_return_arr_0_arr) > 1)
-						{
-							my @return_arr_tmp = unpack($form_return, $response_packet_payload);
-
-							if($form_return_arr_0_arr[0] eq 'a')
-							{
-								my @return_arr = split('', $return_arr_tmp[0]);
-								return \@return_arr;
-							}
-
-							if($form_return_arr_0_arr[0] eq 'Z')
-							{
-								my $return_arr = @return_arr_tmp;
-								return $return_arr;
-							}
-
-							my @return_arr = @return_arr_tmp;
-							return @return_arr;
-						}
-
-						if(scalar(@form_return_arr_0_arr) == 1)
-						{
-							return unpack($form_return, $response_packet_payload);
-						}
-					}
+					return $self->{ipcon}->_dispatch_response($response_packet_payload, $form_return, undef, undef);
 				}
 			}
 		}
@@ -448,13 +282,13 @@ sub get_response_expected
 
 	lock(${$self->{device_lock_ref}});
 
-	if (!defined($self->{response_expected}->{$function_id}))
+	if(!defined($self->{response_expected}->{$function_id}))
 	{
 		croak(Tinkerforge::Error->_new(Tinkerforge::Error->INVALID_FUNCTION_ID, "Function ID $function_id is unknown"));
 	}
 
-	if ($self->{response_expected}->{$function_id} == &_RESPONSE_EXPECTED_ALWAYS_TRUE ||
-	    $self->{response_expected}->{$function_id} == &_RESPONSE_EXPECTED_TRUE)
+	if($self->{response_expected}->{$function_id} == &_RESPONSE_EXPECTED_ALWAYS_TRUE ||
+	   $self->{response_expected}->{$function_id} == &_RESPONSE_EXPECTED_TRUE)
 	{
 		return 1;
 	}
@@ -486,18 +320,18 @@ sub set_response_expected
 
 	lock(${$self->{device_lock_ref}});
 
-	if (!defined($self->{response_expected}->{$function_id}))
+	if(!defined($self->{response_expected}->{$function_id}))
 	{
 		croak(Tinkerforge::Error->_new(Tinkerforge::Error->INVALID_FUNCTION_ID, "Function ID $function_id is unknown"));
 	}
 
-	if ($self->{response_expected}->{$function_id} != &_RESPONSE_EXPECTED_TRUE &&
-	    $self->{response_expected}->{$function_id} != &_RESPONSE_EXPECTED_FALSE)
+	if($self->{response_expected}->{$function_id} != &_RESPONSE_EXPECTED_TRUE &&
+	   $self->{response_expected}->{$function_id} != &_RESPONSE_EXPECTED_FALSE)
 	{
 		croak(Tinkerforge::Error->_new(Tinkerforge::Error->INVALID_FUNCTION_ID, "Response-Exepcted for function ID $function_id cannot be changed"));
 	}
 
-	if ($response_expected)
+	if($response_expected)
 	{
 		$self->{response_expected}->{$function_id} = &_RESPONSE_EXPECTED_TRUE;
 	}
@@ -522,10 +356,10 @@ sub set_response_expected_all
 
 	foreach my $function_id (keys $self->{response_expected})
 	{
-		if ($self->{response_expected}->{$function_id} == &_RESPONSE_EXPECTED_TRUE ||
-		    $self->{response_expected}->{$function_id} == &_RESPONSE_EXPECTED_FALSE)
+		if($self->{response_expected}->{$function_id} == &_RESPONSE_EXPECTED_TRUE ||
+		   $self->{response_expected}->{$function_id} == &_RESPONSE_EXPECTED_FALSE)
 		{
-			if ($response_expected)
+			if($response_expected)
 			{
 				$self->{response_expected}->{$function_id} = &_RESPONSE_EXPECTED_TRUE;
 			}
